@@ -8,12 +8,13 @@ from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,corrSelector,prefitModel
 #from BDMLtools.selector import RFECVSelector
 from BDMLtools.selector import lassoSelector
+from BDMLtools.plotter import BaseWoePlotter
 from BDMLtools.encoder import woeTransformer
 from BDMLtools.tuner import girdTuner,hgirdTuner
 from BDMLtools.tuner import BayesianXGBTuner,BayesianLgbmTuner,shapCheck
 
 
-__version__ = '0.1.6'
+__version__ = '0.1.7'
 
 __all__ = (    
     dtStandardization,
@@ -32,6 +33,7 @@ __all__ = (
     binTree,
     binChi2,
     binKmeans,
+    BaseWoePlotter,
     faSelector,
     stepLogit,
     cardScorer,
